use rocket::{get, post, routes, fs::TempFile};
use rocket::http::Status;
use rocket::serde::json::Json;
#[macro_use] extern crate diesel;

mod data;
mod lib;
mod schema;
use data::{DbConn, FileInfo};


#[get("/")]
fn test() -> &'static str {
    return "Hello, world!";
}


#[get("/<id>")]
async fn download(id: String, conn: DbConn) -> Result<Json<FileInfo>, Status> {
    data::get_file_info(id, conn).await
        .map(|info| Json(info))
        .map_err(|_| Status::InternalServerError)
}


#[post("/upload", data = "<file>")]
async fn upload(mut file: TempFile<'_>, conn: DbConn) -> Result<String, Status> {
    let id = lib::gen_id();
    data::put_file_info(&id, file.name(), conn).await
        .map_err(|_| Status::InternalServerError)?;


    lib::persist_file(&mut file, &id).await
        .map_err(|_| Status::InternalServerError)?;
    Ok(id)
}


#[rocket::launch]
fn rocket() -> _ {
    rocket::build()
        .attach(DbConn::fairing())
        .mount("/", routes![test, upload, download])
}
